/**
 * @file NexUpload.cpp
 *
 * The implementation of uploading tft file for nextion displays. 
 * 
 * Modified to work with ESP8266 and SoftwareSerial
 * @author Ville Vilpas (psoden@gmail.com)
 * @date   2018/2/3
 * @version 0.2.0
 *
 * Original version (a part of https://github.com/itead/ITEADLIB_Arduino_Nextion)
 * @author  Chen Zengpeng (email:<zengpeng.chen@itead.cc>)
 * @date    2016/3/29
 * @copyright 
 * Copyright (C) 2014-2015 ITEAD Intelligent Systems Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify  
 * it under the terms of the GNU General Public License as published by  
 * the Free Software Foundation, version 3.

 *
 * This program is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty of 
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU 
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "ESPNexUploadStream.h"
#include <SoftwareSerial.h>




#define DEBUG_SERIAL_ENABLE


#ifdef DEBUG_SERIAL_ENABLE
#define dbSerialPrint(a)    Serial.print(a)
#define dbSerialPrintln(a)  Serial.println(a)
#define dbSerialBegin(a)    Serial.begin(a)
#else
#define dbSerialPrint(a)    do{}while(0)
#define dbSerialPrintln(a)  do{}while(0)
#define dbSerialBegin(a)    do{}while(0)
#endif


ESPNexUploadStream::ESPNexUploadStream(uint32_t download_baudrate, SoftwareSerial *softSerial)
{
    _download_baudrate = download_baudrate;
    nexSerial = softSerial;
}




bool ESPNexUploadStream::uploadBegin(uint32_t filesize) {
    String temp = "";
    return this->uploadBegin(temp, filesize);
}

bool ESPNexUploadStream::uploadBegin(String &statusMessage, uint32_t filesize)
{	
	_undownloadByte = filesize;
    if(_getBaudrate() == 0)
    {
        dbSerialPrintln("get baudrate error");
        statusMessage = "get baudrate error";
        return false;
    }
    if(!_setDownloadBaudrate(_download_baudrate))
    {
        dbSerialPrintln("modify baudrate error");
        statusMessage = "modifybaudrate error";
        return false;
    }
    dbSerialPrintln("uploadBegin ok\r\n");
    statusMessage = "uploadBegin ok";
    return true;
}

uint16_t ESPNexUploadStream::_getBaudrate(void)
{
    _baudrate = 0;
    uint32_t baudrate_array[7] = {115200,57600,38400,19200,9600,4800,2400};
    for(uint8_t i = 0; i < 7; i++)
    {
		dbSerialPrint("get baudrate try");		
		dbSerialPrintln(baudrate_array[i]);				
        if(_searchBaudrate(baudrate_array[i]))
        {
            _baudrate = baudrate_array[i];
            dbSerialPrintln("get baudrate");
            break;
        }
    }
    return _baudrate;
}


bool ESPNexUploadStream::_searchBaudrate(uint32_t baudrate)
{
    String string = String("");  
    nexSerial->begin(baudrate);
    this->sendCommand("");
    this->sendCommand("connect");
    this->recvRetString(string,500,true);  
	  dbSerialPrintln(string);	
    if(string.indexOf("comok") != -1)
    {
        return 1;
    } 
    return 0;
}

void ESPNexUploadStream::sendCommand(const char* cmd)
{

    while (nexSerial->available())
    {
        nexSerial->read();
    }

    nexSerial->print(cmd);
    nexSerial->write(0xFF);
    nexSerial->write(0xFF);
    nexSerial->write(0xFF);
}

uint16_t ESPNexUploadStream::recvRetString(String &string, uint32_t timeout,bool recv_flag)
{
    uint16_t ret = 0;
    uint8_t c = 0;
    long start;
    bool exit_flag = false;
    start = millis();
    while (millis() - start <= timeout)
    {
        while (nexSerial->available())
        {
            c = nexSerial->read(); 
            if(c == 0)
            {
                continue;
            }
            string += (char)c;
            if(recv_flag)
            {
                if(string.indexOf(0x05) != -1)
                { 
                    exit_flag = true;
                } 
            }
        }
        if(exit_flag)
        {
            break;
        }
    }
    ret = string.length();
    return ret;
}

bool ESPNexUploadStream::_setDownloadBaudrate(uint32_t baudrate)
{
    String string = String(""); 
    String cmd = String("");
    
    String filesize_str = String(_undownloadByte,10);
    String baudrate_str = String(baudrate);
//    String baudrate_str = String(57600);	
    cmd = "whmi-wri " + filesize_str + "," + baudrate_str + ",0";


    this->sendCommand("");
    this->sendCommand(cmd.c_str());


    dbSerialPrintln("Changing baudrate...");
    nexSerial->begin(baudrate);
    this->recvRetString(string,500,true);
    dbSerialPrintln(string);
    if(string.indexOf(0x05) != -1)
    { 
        return 1;
    } 
    return 0;
}

bool ESPNexUploadStream::uploadData(uint8_t data[], uint16_t dataSize)
{
  String string = String("");
  ESP.wdtFeed();

	for(uint16_t j = 0; j < dataSize; j++)
	{
//		Serial.println(j);
		nexSerial->write(data[j]);
	}
	
    this->recvRetString(string,500,true);  
	Serial.println(string);	
	if(string.indexOf(0x05) != -1)
	{ 
		string = "";
} 
	else
	{
		return false;
	}
	
    return true;  
}
